/*******************************************************************************
*   $FILE:  main.c
*   Atmel Corporation:  http://www.atmel.com \n
*   Support email:  touch@atmel.com
******************************************************************************/

/*  License
*   Copyright (c) 2010, Atmel Corporation All rights reserved.
*
*   Redistribution and use in source and binary forms, with or without
*   modification, are permitted provided that the following conditions are met:
*
*   1. Redistributions of source code must retain the above copyright notice,
*   this list of conditions and the following disclaimer.
*
*   2. Redistributions in binary form must reproduce the above copyright notice,
*   this list of conditions and the following disclaimer in the documentation
*   and/or other materials provided with the distribution.
*
*   3. The name of ATMEL may not be used to endorse or promote products derived
*   from this software without specific prior written permission.
*
*   THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
*   WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
*   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
*   SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
*   INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
*   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
*   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
*   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
*   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
*   THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*----------------------------------------------------------------------------
                            compiler information
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
                                include files
----------------------------------------------------------------------------*/
#include <avr/io.h>
#include <avr/interrupt.h>
#define __delay_cycles(n)     __builtin_avr_delay_cycles(n)
#define __enable_interrupt()  sei()
#define F_CPU 16000000UL
#include <util/delay.h>

#include "touch_api.h"
#include "touch.h"
/*----------------------------------------------------------------------------
                            manifest constants
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
                                    macros
----------------------------------------------------------------------------*/

#define GET_SENSOR_STATE(SENSOR_NUMBER) qt_measure_data.qt_touch_status.sensor_states[(SENSOR_NUMBER/8)] & (1 << (SENSOR_NUMBER % 8))
#define GET_ROTOR_SLIDER_POSITION(ROTOR_SLIDER_NUMBER) qt_measure_data.qt_touch_status.rotor_slider_values[ROTOR_SLIDER_NUMBER]

/*----------------------------------------------------------------------------
                            type definitions
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
                                prototypes
----------------------------------------------------------------------------*/
extern void touch_measure();
extern void touch_init( void );
extern void init_system( void );
extern void init_timer_isr(void);
extern void set_timer_period(uint16_t);
/*----------------------------------------------------------------------------
                            Structure Declarations
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
                                    macros
----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
                                global variables
----------------------------------------------------------------------------*/
/* Timer period in msec. */
uint16_t qt_measurement_period_msec = QT_MEASUREMENT_PERIOD_MS;
uint16_t time_ms_inc=0;
/*----------------------------------------------------------------------------
                                extern variables
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
                                static variables
----------------------------------------------------------------------------*/

/* flag set by timer ISR when it's time to measure touch */
volatile uint8_t time_to_measure_touch = 0u;

/* current time, set by timer ISR */
volatile uint16_t current_time_ms_touch = 0u;

/*============================================================================
Name    :   main
------------------------------------------------------------------------------
Purpose :   main code entry point
Input   :   n/a
Output  :   n/a
Notes   :
============================================================================*/
void uart_putchar(unsigned char a)
{
	
	loop_until_bit_is_set(UCSR0A, UDRE0);
	UDR0 = a;
}
void uart_puthex(unsigned char a)
{
	if (a < 10)
		uart_putchar(a+'0');
	else
		uart_putchar(a+'a'-10);
	
}
void uart_putbyte(unsigned char a)
{
	uart_puthex(a >> 4);
	uart_puthex(a & 0x0f);	
}


int main( void )
{

   /* initialise host app, pins, watchdog, etc */
    init_system();

    /* configure timer ISR to fire regularly */
    init_timer_isr();

	/* Initialize Touch sensors */
	touch_init();

	UCSR0B = _BV(TXEN0) | _BV(RXEN0);
	UBRR0 = 103;
	loop_until_bit_is_set(UCSR0A, UDRE0);
	UDR0 = 'a';
	
	TCCR2A |= _BV(COM2A1) | _BV(WGM21) | _BV(WGM20);
	TCCR2B |= _BV(CS20);
	OCR2A = 0;
	
	uint8_t lastvalue = 0;
    /* loop forever */
    for( ; ; )
    {
        touch_measure();
		
		
		DDRD |= _BV(PD7);
		if (qt_measure_data.qt_touch_status.sensor_states[0] & _BV(3))		
			PORTD |= _BV(PD7);
		else
			PORTD &= ~_BV(PD7);
		char str[10];
		sprintf(str, "%d", qt_get_sensor_delta(0));
		//itoa(qt_get_sensor_delta(0), str, 10);
		if (qt_measure_data.qt_touch_status.rotor_slider_values[0] != 0)
		{
			uint8_t newvalue = qt_measure_data.qt_touch_status.rotor_slider_values[0];
			if (newvalue > lastvalue && newvalue < (lastvalue+ 127))
				OCR2A+= ((uint8_t)(newvalue - lastvalue ))/2;
			if (newvalue < lastvalue && newvalue > (lastvalue - 127))
				OCR2A-= ((uint8_t)(lastvalue - newvalue ))/2;
			lastvalue = newvalue;
			//OCR2A = qt_measure_data.qt_touch_status.rotor_slider_values[0];
			//uart_putbyte(qt_measure_data.qt_touch_status.rotor_slider_values[0]);
			//uart_putchar('\r');
			//uart_putchar('\n');
		}
		if ((qt_measure_data.qt_touch_status.sensor_states[0] & _BV(0)) != 0)
		{
			OCR2A = 0;/*
			uint8_t i;
			//for (i = 0; i <=7; i++)
				//uart_putchar(((qt_measure_data.qt_touch_status.sensor_states[1] & _BV(i)) != 0)? '1' : '0');
				uart_putbyte(qt_measure_data.qt_touch_status.sensor_states[0]);
		
			uart_putchar('\r');
			uart_putchar('\n');*/
		}
		//_delay_ms(100);
    /*  Time Non-critical host application code goes here  */
    }
}

